@@ -335,7 +335,7 @@ mspcdsty1 <- function(s,r,id.basis,cnt,int.s,int.r,prec,maxiter,alpha,skip.iter)
                         cd=as.double(cd), as.integer(nxis),
                         as.double(10^lambda*q.wk), as.integer(nxi),
                         as.double(cbind(r.wk,s)), as.integer(nobs),
-                        as.integer(sum(cnt)), as.integer(cnt),
+                        as.double(sum(cnt)), as.double(cnt),
                         as.double(c(int.r.wk,int.s)),
                         as.double(prec), as.integer(maxiter),
                         as.double(.Machine$double.eps), integer(nxis),
@@ -382,7 +382,7 @@ mspcdsty1 <- function(s,r,id.basis,cnt,int.s,int.r,prec,maxiter,alpha,skip.iter)
                         cd=as.double(cd), as.integer(nxis),
                         as.double(10^lambda*q.wk), as.integer(nxi),
                         as.double(cbind(r.wk0,s)), as.integer(nobs),
-                        as.integer(sum(cnt)), as.integer(cnt),
+                        as.double(sum(cnt)), as.double(cnt),
                         as.double(c(int.r.wk0,int.s)),
                         as.double(prec), as.integer(maxiter),
                         as.double(.Machine$double.eps), integer(nxis),
